//****************************************************************************
//	
//	Copyright 2017 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : receive.cpp
//
//	Project	 : PCMCAN Windows Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	06/23/17	  1.0		Original Release	
//
//****************************************************************************

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>  
#include <windows.h>
#include "pcmcanDLL.h"

#define MAJOR_VER   1
#define MINOR_VER   0

#define CHANNEL     1
#define BITRATE     500000

#define IRQ         'Y'  // Y or N
#define HWFILTER    0    // 0 or 1 or 2
#define SWFILTER    0    // 0 or 1

// hw filters
#define ACC_CODES1  0x91A00000  // range 0x12340000
#define ACC_MASKS1  0x0007FFFC  //    to 0x1234ffff
#define ACC_CODES2  0x8000c000  // range 0x10000000
#define ACC_MASKS2  0x3FFF1FFF  //    to 0x1bffffff

// sw filters
#define SWF_LOW_ID  0x00001234
#define SWF_HI_ID   0x00005678

#define RBS_BIT     0x01

#if IRQ == 'N'
void waitForMsg(void)
{
    int dllReturn = 0;
    unsigned int status_reg;
    unsigned int arbit_lost_capture_reg;
    unsigned int error_code_capture_reg;

    do
    {
        dllReturn = ReadStatus(CHANNEL, &status_reg, &arbit_lost_capture_reg, &error_code_capture_reg);

        if (dllReturn)
        {
            printf("Error reading status\n");
            exit(4);
        }

        Sleep(1);
    } while (!(status_reg & RBS_BIT));
}
#endif

int _tmain(int argc, _TCHAR* argv[])
{
    int dllReturn = 0;
    unsigned long pckts_rcvd = 0;
    unsigned long msgID = 0;
    int msgLen = 0;
    unsigned int msgType = 0, data[8] = { 0 };

    printf("PCM-CAN Application : receive\n");
    printf("Version %d.%d\n\n", MAJOR_VER, MINOR_VER);

    dllReturn = InitializeSession();

    if (dllReturn)
    {
        printf("Error initializing session\n");
        exit(dllReturn);
    }

    dllReturn = StartChannel(CHANNEL, BITRATE);

    if (dllReturn)
    {
        printf("Error starting channel\n");
        exit(dllReturn);
    }

#if IRQ == 'N'
    dllReturn = EnableInterrupts(CHANNEL, ALL_IRQ_ENABLE & ~RECEIVE_IRQ_ENBL);

    if (dllReturn)
    {
        printf("Error disabling transmit interrupt\n");
        exit(dllReturn);
    }
#else
    dllReturn = EnableInterrupts(CHANNEL, ALL_IRQ_ENABLE);

    if (dllReturn)
    {
        printf("Error enabling all interrupts\n");
        exit(dllReturn);
    }
#endif

#if HWFILTER == 1
    dllReturn = SetHwFilter(CHANNEL, SINGLE, ACC_CODES1, ACC_MASKS1);

    if (dllReturn)
    {
        printf("Error adding hw filter\n");
        exit(dllReturn);
    }
#elif HWFILTER == 2
    dllReturn = SetHwFilter(CHANNEL, DUAL, ACC_CODES2, ACC_MASKS2);

    if (dllReturn)
    {
        printf("Error adding hw filters\n");
        exit(dllReturn);
    }
#else
    dllReturn = ClearHwFilter(CHANNEL);

    if (dllReturn)
    {
        printf("Error clearing hw filter\n");
        exit(dllReturn);
    }
#endif

#if SWFILTER == 1
    dllReturn = SetSwFilter(CHANNEL, EFF_DATA, SWF_LOW_ID, SWF_HI_ID);

    if (dllReturn)
    {
        printf("Error adding sw filter\n");
        exit(dllReturn);
    }
#else
    dllReturn = ClearSwFilter(CHANNEL);

    if (dllReturn)
    {
        printf("Error clearing sw filter\n");
        exit(dllReturn);
    }
#endif

    while (1)
    {
#if IRQ == 'N'
        waitForMsg();

        dllReturn = ReadDatagram(CHANNEL, &msgID, &msgType, &msgLen, data);
#else

        do
        {
            dllReturn = ReadDatagram(CHANNEL, &msgID, &msgType, &msgLen, data);

            if (dllReturn != SUCCESS && dllReturn != RCV_BUFFER_EMPTY)
            {
                printf("Read error\n");
                exit(dllReturn);
            }

            Sleep(1);
        } while (dllReturn != 0); // 0 indicates message has been read

#endif

        if (msgType == SFF_DATA)
        {
            printf("Received standard data message:\n");
            printf("  ID      = 0x%03x\n", msgID);
            printf("  Length  = %d\n", msgLen);
            for (int i = 0; i < msgLen; i++)
                printf("  Data[%d] = 0x%02x\n", i, data[i]);
            printf("\n");
        }
        else if (msgType == EFF_DATA)
        {
            printf("Received extended data message:\n");
            printf("  ID      = 0x%08x\n", msgID);
            printf("  Length  = %d\n", msgLen);
            for (int i = 0; i < msgLen; i++)
                printf("  Data[i] = 0x%02x\n", data[i]);
            printf("\n");
        }
        else if (msgType == SFF_RTR)
        {
            printf("Received standard remote message:\n");
            printf("  ID      = 0x%03x\n", msgID);
            printf("\n");
        }
        else if (msgType == EFF_RTR)
        {
            printf("Received extended remote message:\n");
            printf("  ID      = 0x%08x\n", msgID);
            printf("\n");
        }

        pckts_rcvd++;
        printf("Packets received : %d\n\n", pckts_rcvd);
    }
    
    CloseSession();

    return 0;
}
